@@ -33,20 +33,19 @@
 /* include test cases' include files here */
 #include "nghttp3_qpack_test.h"
 #include "nghttp3_conn_test.h"
+#include "nghttp3_stream_test.h"
 #include "nghttp3_tnode_test.h"
 #include "nghttp3_http_test.h"
 #include "nghttp3_conv_test.h"
 
 int main(int argc, char **argv) {
   const MunitSuite suites[] = {
-    qpack_suite,
-    conn_suite,
-    tnode_suite,
-    http_suite,
-    {NULL, NULL, NULL, 0, MUNIT_SUITE_OPTION_NONE},
+    qpack_suite, conn_suite, stream_suite, tnode_suite, http_suite, {0},
   };
   const MunitSuite suite = {
-    "", NULL, suites, 1, MUNIT_SUITE_OPTION_NONE,
+    .prefix = "",
+    .suites = suites,
+    .iterations = 1,
   };
 
   return munit_suite_main(&suite, NULL, argc, argv);
